module parameters
  implicit none
contains

  subroutine clausius_clapeyron_equation(pv0,t0,t,L,R,pv) 

   !-クラウジウスクラペイロンの公式で基準温度における飽和水上気圧を求める

    implicit none
    real(8),intent(in) :: pv0, t0, t, L, R
    real(8),intent(out):: pv

    ! 簡易化したclausius clapeyron の公式
    ! 地表面における飽和水上気圧を求める

    pv=pv0*exp(-L/R/t)

  end subroutine clausius_clapeyron_equation

  subroutine surface_pressure(pv,pn0,ps)
    implicit none
    real(8),intent(in):: pv, pn0
    real(8),intent(out):: ps

    ! 表面圧力を計算
    ! 比凝結気体の圧力と凝結気体(水蒸気)圧力の合計によって求める

    ps=pn0+pv

  end subroutine surface_pressure

  subroutine temperature_distribution(tmax,ps,pv,pv0,t,L,cpv,cpn,R,RA,p,tp,xv,sigm)

   ! 湿潤断熱勾配を用いて温度の鉛直分布を求める

    implicit none
    integer i,k
    integer,intent(in)::tmax                             ! メインプログラムで定義
    real(8) dp                       !圧力の変化量
    real(8) logsigmt                 ! 表面圧力に対する大気上端圧力の常用対数
    real(8) dlogsigm                 ! 表面圧力に対する大気圧力の差
    real(8) ddlogsigm                ! 半整数値を求めるための変数  
    real(8) k1,k2,pvvc,xvc,xnc                    ! 変数
    real(8) p(0:tmax)        ! 全圧
    real(8) xn(0:tmax)       ! 全圧に対する凝結気体の圧力
    real(8) pvv(0:tmax)      ! 飽和水蒸気圧
    real(8) sig(0:tmax)      ! 表面圧力に対する任意の高さにおける圧力
    real(8) logsigm(0:tmax)  ! sig に常用対数をとったもの
    real(8) logsigm_kako(0:tmax)
    real(8) sigm_kako(0:tmax),p_kako(0:tmax)
    real(8),intent(in)::ps,t,L,R,RA,pv,pv0,cpv,cpn       ! メインプログラムで定義
    real(8),intent(out)::tp(0:tmax),xv(0:tmax),sigm(0:tmax) ! メインプログラムで定義


    logsigmt=-6.0d0              ! 大気上端を定義
    dlogsigm=logsigmt/real(tmax) ! 鉛直座標のステップの定義
    ddlogsigm=dlogsigm*0.5       ! 半整数値幅の定義
    
    logsigm(0)=ddlogsigm         ! 初期値の定義
 
    ! すべてのlog10(sig)を計算していく

    do i=1,tmax

      logsigm(i)=logsigm(i-1)+dlogsigm

    end do

    ! sig から圧力を計算する
    ! sig=(ある高さの圧力/表面圧力) であることを用いる

    do i=0,tmax
      sigm(i)=10.0d0**logsigm(i)
    end do


    p(:)=ps*sigm(:)

    !-- 修正オイラー法によって温度分布を求める

    pvv(0)=pv                ! 表面飽和水蒸気圧
    xv(0)=pvv(0)/p(0)        ! 表面比湿(表面圧力に対する凝結大気の分圧)
    xn(0)=(p(0)-pvv(0))/p(0) ! 表面圧力に対する乾燥大気の分圧
    tp(0)=t                  ! 表面温度
 
   do i=0,tmax-1

      dp=p(i)-p(i+1)

      k1=((R*tp(i))/(p(i)*cpn)+(xv(i)/xn(i))*(l/(p(i)*cpn)))/&
      &(xn(i)+xv(i)*(cpv/cpn)+(xv(i)/xn(i))*(l**2/(R*tp(i)**2*cpn)))

      pvvc=pv0*exp(-L/R/(tp(i)-dp*k1))
      xvc=pvvc/p(i+1)
      xnc=(p(i+1)-pvvc)/p(i+1)
 
      k2=((R*(tp(i)-k1*dp))/((p(i)-dp)*cpn)+(xvc/xnc)*(l/((p(i)-dp)*cpn)))/&
      &(xnc+xvc*(cpv/cpn)+(xvc/xnc)*(l**2/(R*(tp(i)-k1*dp)**2*cpn)))
 

      tp(i+1)=tp(i)-dp/2.0*(k1+k2)

      !-- 求めた温度から飽和水上気圧とモル分率を計算 

      pvv(i+1)=pv0*exp(-L/R/tp(i+1))
      xv(i+1)=pvv(i+1)/p(i+1)
      xn(i+1)=(p(i+1)-pvv(i+1))/p(i+1)
 

    end do
 
  end subroutine temperature_distribution
 
  subroutine temperature_distribution_tau(xv,tmax,g,kv,kn,tp,p,tau)
    implicit none
    integer,intent(in)::tmax
    real(8),intent(in)::kv,kn,g
    real(8),intent(in)::tp(0:tmax),p(0:tmax),xv(0:tmax)
    real(8),intent(out)::tau(0:tmax)
    real(8) dp,xn(0:tmax)
    integer i

    !-- 光学的厚さの計算

    tau(tmax)=0.0d0

    xn(tmax)=(1.0-xv(tmax))
    do i=tmax,1,-1
      xn(i-1)=1.0-xv(i-1)
      dp=p(i-1)-p(i)
      tau(i-1)=tau(i)+1.5*kv*(xv(i-1)+xv(i))*0.5d0/g*dp+1.5*kn*(xn(i-1)+xn(i))*0.5d0/g*dp
   end do  

  end subroutine temperature_distribution_tau

  subroutine flux_distribution(tmax,pi,sig,tp,tau,fnet,sigm,bound)
    implicit none
    integer,intent(in)::tmax
    integer,intent(out)::bound
    real(8) pi,sig,b1(tmax),b2(tmax),btau,c,dc,btop
    real(8),intent(in)::tp(0:tmax),tau(0:tmax),sigm(0:tmax)
    real(8),intent(inout)::fnet(tmax)
    integer i,k
    
    c=0.1
 
    !--下端から求めたい高度まで通過してきたフラックス--
    do k=1,tmax
      b1(k)=0.0d0
      b2(k)=0.0d0
      
    !--大気上端における放射フラックス
      btau=sig*tp(tmax)**4*exp(-(tau(k)-tau(tmax)))

      do i=1,k
        dc=tau(i-1)-tau(i)
       if (dc < c) then
          b1(k)=b1(k)+(sig*tp(i-1)**4-sig*tp(i)**4)*(exp(-(tau(i-1)-tau(k)))+exp(-(tau(i)-tau(k))))/2.0
        else if (dc >= c) then
          b1(k)=b1(k)-(sig*tp(i-1)**4-sig*tp(i)**4)/(tau(i-1)-tau(i))*(exp(-(tau(i-1)-tau(k)))-exp(-(tau(i)-tau(k))))
        end if
      end do

      do i=k+1,tmax
        dc=tau(i-1)-tau(i)
        if (dc < c) then
          b2(k)=b2(k)+(sig*tp(i-1)**4-sig*tp(i)**4)*(exp(-(tau(k)-tau(i-1)))+exp(-(tau(k)-tau(i))))/2.0
        else if (dc >= c) then
          b2(k)=b2(k)+(sig*tp(i-1)**4-sig*tp(i)**4)/(tau(i-1)-tau(i))*(exp(-(tau(k)-tau(i-1)))-exp(-(tau(k)-tau(i))))
        end if 
      end do
      fnet(k)=btau+b1(k)+b2(k)
      end do


  end subroutine flux_distribution

  subroutine toropopause(fnet,tau,bound,sig,tp,tmax)
    implicit none
    integer k
    integer,intent(in)::tmax
    integer,intent(inout)::bound
    real(8),intent(in)::fnet(tmax),tau(0:tmax),sig,tp(0:tmax)

    !-- 圏界面の目星をつける

    do k=tmax,1,-1
      if(fnet(k) - fnet(k-1) > 0.0001)then
        bound=k 
        exit
      end if
    end do    

    !-- 微調整
   
    do while ((fnet(bound)/2.0*(tau(bound)+1.0)/sig)**(1.0/4.0)-tp(bound) > 0)
      bound=bound-1
    end do


    if (abs(fnet(bound+1)/2.0*(tau(bound+1)+1.0)-sig*tp(bound+1)**4) < abs(fnet(bound)/2.0*(tau(bound)+1.0)-sig*tp(bound)**4)) then
      bound=bound+1
    end if

! 圏界面の決め方(古い方)

!    do k=0,tmax
!      if(fnet(k) > fnet(k+1))then
!        bound=k 
!        exit
!      end if
!    end do    

!    write(*,*)'17',(fnet(17)/2.0*(tau(17)+1.0)/sig)**(1.0/4.0)-tp(17)
   
!    do while ((fnet(bound)/2.0*(tau(bound)+1.0)/sig)**(1.0/4.0)-tp(bound) > 0)

!      write(*,*)bound
!      write(*,*)"bound=",bound,"sabun=",(fnet(bound)/2.0*(tau(bound)+1.0)/sig)**(1.0/4.0)-tp(bound)
!      write(*,*)(fnet(bound)/2.0*(tau(bound)+1.0)/sig)**(1.0/4.0)-tp(bound)
!      bound=bound-1
!    end do

!      write(*,*)"bound=",bound,"sabun=",(fnet(bound)/2.0*(tau(bound)+1.0)/sig)**(1.0/4.0)-tp(bound)

!    write(*,*)fnet(bound)/2.0*(tau(bound)+1.0)-sig*tp(bound)**4
!    write(*,*)(fnet(bound)/2.0*(tau(bound)+1.0)/sig)**(1.0/4.0)-tp(bound)

!    if (abs(fnet(bound+1)/2.0*(tau(bound+1)+1.0)-sig*tp(bound+1)**4) < abs(fnet(bound)/2.0*(tau(bound)+1.0)-sig*tp(bound)**4)) then
!      bound=bound+1
!    end if

!    if(fnet(bound) > fnet(bound+1))then
!      bound=bound-1
!    end if

!    write(*,*)"last_bound=",bound

  end subroutine toropopause


  subroutine stratosphere(L,R,pv0,xv,xvc,kv,kn,g,p,tmax,sig,sigm,tp,fnet,tau,bound)
    implicit none
    integer,intent(in)::tmax,bound
    integer i,n
    real(8),intent(in)::sig,sigm(0:tmax),p(0:tmax),kv,kn,g,xv(0:tmax),L,R,pv0
    real(8) dc,b1(bound:tmax-1),ftop(bound:tmax-1),c,btautop,dp,pv
    real(8),intent(inout)::tp(0:tmax),fnet(tmax),tau(0:tmax),xvc

  !-- 成層圏の温度を求める
  !-- もし bound が tmax 以上であれば成層圏を考えない
  if (bound < tmax)then

    write(*,*)'fnet(bound)',fnet(bound)

      pv=pv0*exp(-L/R/tp(bound))
      c=pv/(p(bound)-pv) !-- 成層圏の比湿は一定
      xvc=c/(1+c)

    !-- 光学的厚さを再計算

    do n=tmax,bound+1,-1 
      dp=p(n-1)-p(n)
      tau(n-1)=tau(n)+1.5*kv*xvc/g*dp+1.5*kn*(1-xvc)/g*dp
    end do

    !-- 放射平衡の条件から温度を求める

    do n=bound,tmax
      tp(n)=(fnet(bound)/(2.0*sig)*(tau(n)+1.0))**(0.25d0)
    end do

  end if  

  end subroutine stratosphere

  subroutine heating_rate(tau,k,tmax,p,sigm,xv,xvc,cp,fnet,heat_rate,bound)
    
    implicit none
    integer,intent(in):: tmax,bound
    integer i,n
    real(8)cpt
    real(8),intent(in)::cp
    real(8),intent(in)::p(0:tmax),sigm(0:tmax),xv(0:tmax),xvc,fnet(tmax),k,tau(0:tmax)
    real(8),intent(out):: heat_rate(0:tmax-1)

    ! 加熱率の計算
    ! 定圧モル比熱をmol当たりからkg当たりに変換
    cpt=cp/18.0d0*10**(3.0)

    do i=0,bound-1
   
      heat_rate(i)=k*xv(i)/cpt*(fnet(i)-fnet(i+1))/(tau(i)-tau(i+1))

    end do

    do i=bound,tmax-1

      heat_rate(i)=k*xvc/cpt*(fnet(i)-fnet(i+1))/(tau(i)-tau(i+1))
   
    end do

  end subroutine heating_rate

  subroutine heating_rate2(bound,tmax,p,sigm,xv,cp,fneth,heat_rate)
    
    implicit none
    integer,intent(in):: tmax,bound
    integer i,n
    real(8),intent(in)::p(0:tmax),sigm(0:tmax),xv(0:tmax),cp,fneth(0:tmax)
    real(8),intent(out):: heat_rate(0:tmax-1)

    ! 加熱率の計算

    do i=0,bound-1

      heat_rate(i)=xv(i)/cp*(fneth(i)-fneth(i+1))/(p(i)-p(i+1))

    end do

  end subroutine heating_rate2



end module parameters


!-- ここからメインプログラム ---
 
program gray_atmosphere
  use parameters
  implicit none
  integer tmax,bound,i,nmax,j,z,zmax,maxnum,minnum,w,lmax
  real(8) pi, e, pv0, t0, t, L, RA, pv, pn0, ps, g, k,ky,dk, sig,cpn,cpv,R,dt,tspan,sun,xvc,tall(7),dte,k2
  real(8),allocatable:: p(:),tp(:),xv(:),tau(:),fnet(:),fneth(:),sigm(:),heat_rate(:),t_ave(:)

!--計算回数--
  tmax=120

!--基準温度での飽和水蒸気圧--
  pv0=1.4d0*10.0**11.0
!--基準となる温度--
  t0=273.16d0 ! [K]
!--水の潜熱--
!  L=2.4253d0*10**6 ! [J/Kg]
  L=43655.0d0 ! [J/mol]
!--水蒸気の気体定数--
  RA=461.9d0 ! [J/Kg/K]
!--飽和水上気圧を求めたい温度-
  t=250.0d0 ! [K]
!--大気下端における乾燥大気の分圧--
  pn0=10.0d0**5.0d0 ! [Pa]
!--重力加速度--
  g=9.8d0 ! [m/s^2]
!--水蒸気の吸収係数--
  k=0.01d0 ! [m^2/kg]
!--乾燥大気の吸収係数--
  k2=0.0d0
!--ステファンボルツマン定数--
  sig=5.67d0*10.0d0**(-8.0d0) ! [W/m^2/K^4]
!--円周率--
  pi=2.0d0*acos(0.0d0)
!--気体定数--
  R=8.314d0
!--凝結物質の定圧モル比熱--
  cpv=4.0d0*8.314d0
!--比凝結物質の定圧モル比熱
  cpn=3.5d0*8.314d0


!----- 変化させる変数 ------

  ! 地表面温度
  t=250
!  t=300
!  t=350
!  t=400
!  t=450
!  t=500
!  t=550

  ! 比凝結成分の吸収係数(適宜代入)
  k2=0.005

!---------------------------
 

  allocate(xv(0:tmax),sigm(0:tmax),p(0:tmax),tp(0:tmax),tau(0:tmax),fnet(tmax),heat_rate(0:tmax-1),fneth(0:tmax),t_ave(zmax)) 

  write(*,*)''
  write(*,*)'parameters'
  write(*,*)''
  write(*,*)'RA=',RA
  write(*,*)'t0=',t0
  write(*,*)'L=',L
  write(*,*)'pv0',pv0
  write(*,*)'t',t
  write(*,*)'pn0',pn0
  write(*,*)'g',g
  write(*,*)'k',k
  write(*,*)'sig',sig
  write(*,*)'pi',pi
  write(*,*)'R=',R
  write(*,*)'cpv',cpv
  write(*,*)'cpn',cpn
  write(*,*)''
 
  write(*,*)'results'
  write(*,*)''

 
  !-- 出力ファイルを定義 
  !-- 大気の鉛直構造に関するファイルのみを用意している

  !-- 温度鉛直分布用

  open(100,file="temp_t250.dat",status="unknown")
!  open(100,file="temp_t300.dat",status="unknown")
!  open(100,file="temp_t350.dat",status="unknown")
!  open(100,file="temp_t400.dat",status="unknown")
!  open(100,file="temp_t450.dat",status="unknown")
!  open(100,file="temp_t500.dat",status="unknown")
!  open(100,file="temp_t550.dat",status="unknown")

  !-- 正味の上向き放射フラックス鉛直分布用

  open(200,file="flux_t250.dat",status="unknown")
!  open(200,file="flux_t300.dat",status="unknown")
!  open(200,file="flux_t350.dat",status="unknown")
!  open(200,file="flux_t400.dat",status="unknown")
!  open(200,file="flux_t450.dat",status="unknown")
!  open(200,file="flux_t500.dat",status="unknown")
!  open(200,file="flux_t550.dat",status="unknown")

  !-- 加熱率鉛直分布用

  open(300,file="heatrate_t250.dat",status="unknown")
!  open(300,file="heatrate_t300.dat",status="unknown")
!  open(300,file="heatrate_t350.dat",status="unknown")
!  open(300,file="heatrate_t400.dat",status="unknown")
!  open(300,file="heatrate_t450.dat",status="unknown")
!  open(300,file="heatrate_t500.dat",status="unknown")
!  open(300,file="heatrate_t550.dat",status="unknown")

  !-- 光学的厚さの温度分布用

  open(400,file="tau_t250.dat",status="unknown")
!  open(400,file="tau_t300.dat",status="unknown")
!  open(400,file="tau_t350.dat",status="unknown")
!  open(400,file="tau_t400.dat",status="unknown")
!  open(400,file="tau_t450.dat",status="unknown")
!  open(400,file="tau_t500.dat",status="unknown")
!  open(400,file="tau_t550.dat",status="unknown")
 

 
  !-- サブルーチンをコール

  call clausius_clapeyron_equation(pv0,t0,t,L,R,pv)
  write(*,*)'pv',pv

  call surface_pressure(pn0,pv,ps)
  write(*,*)'ps=',ps

  call temperature_distribution(tmax,ps,pv,pv0,t,L,cpv,cpn,R,RA,p,tp,xv,sigm)

  call temperature_distribution_tau(xv,tmax,g,k,k2,tp,p,tau)

  call flux_distribution(tmax,pi,sig,tp,tau,fnet,sigm,bound)

  call toropopause(fnet,tau,bound,sig,tp,tmax)

  call stratosphere(L,R,pv0,xv,xvc,k,k2,g,p,tmax,sig,sigm,tp,fnet,tau,bound)

  call flux_distribution(tmax,pi,sig,tp,tau,fnet,sigm,bound)

  call heating_rate(tau,k,tmax,p,sigm,xv,xvc,cpv,fnet,heat_rate,bound)

 
  do i=1,tmax-1

    ! 作図用に値を補正する
    heat_rate(i)=heat_rate(i)*(10**4)
 
  end do

  !-- 温度, 正味の上向き放射フラックス, 加熱率の鉛直分布を出力

  do i=1,tmax
  
    write(100,*)tp(i),log10(sigm(i))
    write(200,*)fnet(i),log10(sigm(i))
    write(300,*)heat_rate(i),log10(sigm(i))

  end do

  !　光学的厚さを出力

  do j=0,tmax
     write(100,*) tau(j),log10(sigm(j))
  end do


end program gray_atmosphere
